package sun.security.provider;

import java.text.MessageFormat;
import java.io.FileInputStream;
import java.io.InputStream;
import java.util.Locale;
import java.util.LinkedList;
import java.io.BufferedReader;
import java.io.Reader;
import java.security.PrivilegedAction;
import javax.security.auth.AuthPermission;
import java.util.Iterator;
import java.io.FileNotFoundException;
import sun.security.util.ResourcesMgr;
import java.io.InputStreamReader;
import java.net.MalformedURLException;
import java.io.File;
import sun.security.util.PropertyExpander;
import java.util.HashMap;
import java.security.Security;
import java.security.PrivilegedActionException;
import java.security.AccessController;
import java.security.URIParameter;
import java.security.PrivilegedExceptionAction;
import java.net.URI;
import java.io.IOException;
import sun.security.util.Debug;
import java.io.StreamTokenizer;
import java.util.List;
import java.util.Map;
import java.net.URL;
import javax.security.auth.login.ConfigurationSpi;
import javax.security.auth.login.AppConfigurationEntry;
import javax.security.auth.login.Configuration;

public final class ConfigFile extends Configuration
{
    private final ConfigFile.Spi spi;
    
    public ConfigFile() {
        this.spi = new ConfigFile.Spi();
    }
    
    @Override
    public AppConfigurationEntry[] getAppConfigurationEntry(String s) {
        return this.spi.engineGetAppConfigurationEntry(s);
    }
    
    @Override
    public synchronized void refresh() {
        this.spi.engineRefresh();
    }
    
    public static final class Spi extends ConfigurationSpi
    {
        private URL url;
        private boolean expandProp;
        private Map<String, List<AppConfigurationEntry>> configuration;
        private int linenum;
        private StreamTokenizer st;
        private int lookahead;
        private static Debug debugConfig;
        private static Debug debugParser;
        
        public Spi() {
            this.expandProp = true;
            try {
                this.init();
            }
            catch (IOException cause) {
                throw new SecurityException(cause);
            }
        }
        
        public Spi(URI uri) {
            this.expandProp = true;
            try {
                this.url = uri.toURL();
                this.init();
            }
            catch (IOException cause) {
                throw new SecurityException(cause);
            }
        }
        
        public Spi(Configuration.Parameters parameters) throws IOException {
            this.expandProp = true;
            try {
                AccessController.doPrivileged(new PrivilegedExceptionAction<Void>() {
                    @Override
                    public Void run() throws IOException {
                        if (parameters == null) {
                            ConfigFile.Spi.access$000(ConfigFile.Spi.this);
                        }
                        else {
                            if (!(parameters instanceof URIParameter)) {
                                throw new IllegalArgumentException("Unrecognized parameter: " + parameters);
                            }
                            ConfigFile.Spi.this.url = ((URIParameter)parameters).getURI().toURL();
                            ConfigFile.Spi.access$000(ConfigFile.Spi.this);
                        }
                        return null;
                    }
                });
            }
            catch (PrivilegedActionException ex) {
                throw (IOException)ex.getException();
            }
        }
        
        private void init() throws IOException {
            boolean n = false;
            String anObject = Security.getProperty("policy.expandProperties");
            if (anObject == null) {
                anObject = System.getProperty("policy.expandProperties");
            }
            if ("false".equals(anObject)) {
                this.expandProp = false;
            }
            HashMap configuration = new HashMap();
            if (this.url != null) {
                if (ConfigFile.Spi.debugConfig != null) {
                    ConfigFile.Spi.debugConfig.println("reading " + this.url);
                }
                this.init(this.url, configuration);
                this.configuration = configuration;
                return;
            }
            else {
                if ("true".equalsIgnoreCase(Security.getProperty("policy.allowSystemProperty"))) {
                    String pathname = System.getProperty("java.security.auth.login.config");
                    if (pathname != null) {
                        boolean n2 = false;
                        if (pathname.startsWith("=")) {
                            n2 = true;
                            pathname = pathname.substring(1);
                        }
                        try {
                            pathname = PropertyExpander.expand(pathname);
                        }
                        catch (PropertyExpander.ExpandException ex) {
                            throw this.ioException("Unable.to.properly.expand.config", new Object[] { pathname });
                        }
                        Object url;
                        try {
                            url = new URL(pathname);
                        }
                        catch (MalformedURLException ex2) {
                            File o = new File(pathname);
                            if (o.exists()) {
                                url = o.toURI().toURL();
                            }
                            else {
                                throw this.ioException("extra.config.No.such.file.or.directory.", new Object[] { pathname });
                            }
                        }
                        if (ConfigFile.Spi.debugConfig != null) {
                            ConfigFile.Spi.debugConfig.println("reading " + url);
                        }
                        this.init(url, configuration);
                        n = true;
                        if (!(n2 == 0)) {
                            if (ConfigFile.Spi.debugConfig != null) {
                                ConfigFile.Spi.debugConfig.println("overriding other policies!");
                            }
                            this.configuration = configuration;
                            return;
                        }
                    }
                }
                int i;
                String spec;
                for (i = 1; (spec = Security.getProperty("login.config.url." + i)) != null; ++i) {
                    try {
                        spec = PropertyExpander.expand(spec).replace(File.separatorChar, '/');
                        if (ConfigFile.Spi.debugConfig != null) {
                            ConfigFile.Spi.debugConfig.println("\tReading config: " + spec);
                        }
                        this.init(new URL(spec), configuration);
                        n = true;
                    }
                    catch (PropertyExpander.ExpandException ex3) {
                        throw this.ioException("Unable.to.properly.expand.config", new Object[] { spec });
                    }
                }
                if (n == 0 && i == 1 && spec == null) {
                    if (ConfigFile.Spi.debugConfig != null) {
                        ConfigFile.Spi.debugConfig.println("\tReading Policy from ~/.java.login.config");
                    }
                    String string = System.getProperty("user.home") + File.separatorChar + ".java.login.config";
                    if (new File(string).exists()) {
                        this.init(new File(string).toURI().toURL(), configuration);
                    }
                }
                this.configuration = configuration;
                return;
            }
        }
        
        private void init(URL url, Map<String, List<AppConfigurationEntry>> map) throws IOException {
            try (final InputStreamReader inputStreamReader = new InputStreamReader(this.getInputStream(url), "UTF-8")) {
                this.readConfig(inputStreamReader, map);
            }
            catch (FileNotFoundException ex) {
                if (ConfigFile.Spi.debugConfig != null) {
                    ConfigFile.Spi.debugConfig.println(ex.toString());
                }
                throw new IOException(ResourcesMgr.getString("Configuration.Error.No.such.file.or.directory", "sun.security.util.AuthResources"));
            }
        }
        
        public AppConfigurationEntry[] engineGetAppConfigurationEntry(String s) {
            List<Object> list = null;
            synchronized (this.configuration) {
                list = (List)this.configuration.get(s);
            }
            if (list == null || list.size() == 0) {
                return null;
            }
            AppConfigurationEntry[] array = new AppConfigurationEntry[list.size()];
            Iterator<Object> iterator = list.iterator();
            int n = 0;
            while (iterator.hasNext()) {
                AppConfigurationEntry appConfigurationEntry = (AppConfigurationEntry)iterator.next();
                array[n] = new AppConfigurationEntry(appConfigurationEntry.getLoginModuleName(), appConfigurationEntry.getControlFlag(), appConfigurationEntry.getOptions());
                ++n;
            }
            return array;
        }
        
        public synchronized void engineRefresh() {
            SecurityManager securityManager = System.getSecurityManager();
            if (securityManager != null) {
                securityManager.checkPermission(new AuthPermission("refreshLoginConfiguration"));
            }
            AccessController.doPrivileged(new PrivilegedAction<Void>() {
                @Override
                public Void run() {
                    try {
                        ConfigFile.Spi.access$000(ConfigFile.Spi.this);
                    }
                    catch (IOException cause) {
                        throw new SecurityException(cause.getLocalizedMessage(), cause);
                    }
                    return null;
                }
            });
        }
        
        private void readConfig(Reader reader, Map<String, List<AppConfigurationEntry>> map) throws IOException {
            this.linenum = 1;
            if (!(reader instanceof BufferedReader)) {
                reader = new BufferedReader(reader);
            }
            (this.st = new StreamTokenizer(reader)).quoteChar(34);
            this.st.wordChars(36, 36);
            this.st.wordChars(95, 95);
            this.st.wordChars(45, 45);
            this.st.wordChars(42, 42);
            this.st.lowerCaseMode(false);
            this.st.slashSlashComments(true);
            this.st.slashStarComments(true);
            this.st.eolIsSignificant(true);
            this.lookahead = this.nextToken();
            while (this.lookahead != -1) {
                this.parseLoginEntry(map);
            }
        }
        
        private void parseLoginEntry(Map<String, List<AppConfigurationEntry>> map) throws IOException {
            LinkedList<AppConfigurationEntry> list = new LinkedList();
            String sval = this.st.sval;
            this.lookahead = this.nextToken();
            if (ConfigFile.Spi.debugParser != null) {
                ConfigFile.Spi.debugParser.println("\tReading next config entry: " + sval);
            }
            this.match("{");
            while (!this.peek("}")) {
                String match = this.match("module class name");
                String upperCase = this.match("controlFlag").toUpperCase(Locale.ENGLISH);
                int n = -1;
                switch (upperCase.hashCode()) {
                    case 389487519: {
                        if (upperCase.equals("REQUIRED")) {
                            n = 0;
                            break;
                        }
                        break;
                    }
                    case -810754599: {
                        if (upperCase.equals("REQUISITE")) {
                            n = 1;
                            break;
                        }
                        break;
                    }
                    case -848090850: {
                        if (upperCase.equals("SUFFICIENT")) {
                            n = 2;
                            break;
                        }
                        break;
                    }
                    case 703609696: {
                        if (upperCase.equals("OPTIONAL")) {
                            n = 3;
                            break;
                        }
                        break;
                    }
                }
                AppConfigurationEntry.LoginModuleControlFlag controlFlag;
                switch (n) {
                    case 0: {
                        controlFlag = AppConfigurationEntry.LoginModuleControlFlag.REQUIRED;
                        break;
                    }
                    case 1: {
                        controlFlag = AppConfigurationEntry.LoginModuleControlFlag.REQUISITE;
                        break;
                    }
                    case 2: {
                        controlFlag = AppConfigurationEntry.LoginModuleControlFlag.SUFFICIENT;
                        break;
                    }
                    case 3: {
                        controlFlag = AppConfigurationEntry.LoginModuleControlFlag.OPTIONAL;
                        break;
                    }
                    default: {
                        throw this.ioException("Configuration.Error.Invalid.control.flag.flag", new Object[] { upperCase });
                    }
                }
                HashMap options = new HashMap();
                while (!this.peek(";")) {
                    String match2 = this.match("option key");
                    this.match("=");
                    try {
                        options.put(match2, this.expand(this.match("option value")));
                    }
                    catch (PropertyExpander.ExpandException ex) {
                        throw new IOException(ex.getLocalizedMessage());
                    }
                }
                this.lookahead = this.nextToken();
                if (ConfigFile.Spi.debugParser != null) {
                    ConfigFile.Spi.debugParser.println("\t\t" + match + ", " + upperCase);
                    for (String str : options.keySet()) {
                        ConfigFile.Spi.debugParser.println("\t\t\t" + str + "=" + (String)options.get(str));
                    }
                }
                list.add(new AppConfigurationEntry(match, controlFlag, options));
            }
            this.match("}");
            this.match(";");
            if (map.containsKey(sval)) {
                throw this.ioException("Configuration.Error.Can.not.specify.multiple.entries.for.appName", new Object[] { sval });
            }
            map.put(sval, list);
            return;
        }
        
        private String match(String s) throws IOException {
            String sval = null;
            switch (this.lookahead) {
                case -1: {
                    throw this.ioException("Configuration.Error.expected.expect.read.end.of.file.", new Object[] { s });
                }
                case -3:
                case 34: {
                    if (s.equalsIgnoreCase("module class name") || s.equalsIgnoreCase("controlFlag") || s.equalsIgnoreCase("option key") || s.equalsIgnoreCase("option value")) {
                        sval = this.st.sval;
                        this.lookahead = this.nextToken();
                        break;
                    }
                    throw this.ioException("Configuration.Error.Line.line.expected.expect.found.value.", new Object[] { new Integer(this.linenum), s, this.st.sval });
                }
                case 123: {
                    if (s.equalsIgnoreCase("{")) {
                        this.lookahead = this.nextToken();
                        break;
                    }
                    throw this.ioException("Configuration.Error.Line.line.expected.expect.", new Object[] { new Integer(this.linenum), s, this.st.sval });
                }
                case 59: {
                    if (s.equalsIgnoreCase(";")) {
                        this.lookahead = this.nextToken();
                        break;
                    }
                    throw this.ioException("Configuration.Error.Line.line.expected.expect.", new Object[] { new Integer(this.linenum), s, this.st.sval });
                }
                case 125: {
                    if (s.equalsIgnoreCase("}")) {
                        this.lookahead = this.nextToken();
                        break;
                    }
                    throw this.ioException("Configuration.Error.Line.line.expected.expect.", new Object[] { new Integer(this.linenum), s, this.st.sval });
                }
                case 61: {
                    if (s.equalsIgnoreCase("=")) {
                        this.lookahead = this.nextToken();
                        break;
                    }
                    throw this.ioException("Configuration.Error.Line.line.expected.expect.", new Object[] { new Integer(this.linenum), s, this.st.sval });
                }
                default: {
                    throw this.ioException("Configuration.Error.Line.line.expected.expect.found.value.", new Object[] { new Integer(this.linenum), s, this.st.sval });
                }
            }
            return sval;
        }
        
        private boolean peek(String s) {
            switch (this.lookahead) {
                case 44: {
                    return s.equalsIgnoreCase(",");
                }
                case 59: {
                    return s.equalsIgnoreCase(";");
                }
                case 123: {
                    return s.equalsIgnoreCase("{");
                }
                case 125: {
                    return s.equalsIgnoreCase("}");
                }
                default: {
                    return false;
                }
            }
        }
        
        private int nextToken() throws IOException {
            int nextToken;
            while ((nextToken = this.st.nextToken()) == 10) {
                ++this.linenum;
            }
            return nextToken;
        }
        
        private InputStream getInputStream(URL obj) throws IOException {
            if ("file".equalsIgnoreCase(obj.getProtocol())) {
                try {
                    return obj.openStream();
                }
                catch (Exception ex) {
                    String name = obj.getPath();
                    if (obj.getHost().length() > 0) {
                        name = "//" + obj.getHost() + name;
                    }
                    if (ConfigFile.Spi.debugConfig != null) {
                        ConfigFile.Spi.debugConfig.println("cannot read " + obj + ", try " + name);
                    }
                    return new FileInputStream(name);
                }
            }
            else {
                return obj.openStream();
            }
        }
        
        private String expand(String value) throws PropertyExpander.ExpandException, IOException {
            if (value.isEmpty()) {
                return value;
            }
            if (!this.expandProp) {
                return value;
            }
            String expand = PropertyExpander.expand(value);
            if (expand == null || expand.length() == 0) {
                throw this.ioException("Configuration.Error.Line.line.system.property.value.expanded.to.empty.value", new Object[] { new Integer(this.linenum), value });
            }
            return expand;
        }
        
        private IOException ioException(String s, Object... obj) {
            return new IOException(new MessageFormat(ResourcesMgr.getString(s, "sun.security.util.AuthResources")).format(obj));
        }
        
        static void access$000(ConfigFile.Spi spi) throws IOException {
            spi.init();
        }
        
        static {
            ConfigFile.Spi.debugConfig = Debug.getInstance("configfile");
            ConfigFile.Spi.debugParser = Debug.getInstance("configparser");
        }
    }
}
